from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.wrapped_task import WrappedTask
from ...models.wrapped_task_create import WrappedTaskCreate
from ...types import Response


def _get_kwargs(
    *,
    body: WrappedTaskCreate,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/tasks",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[WrappedTask]:
    if response.status_code == 200:
        response_200 = WrappedTask.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[WrappedTask]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedTaskCreate,
) -> Response[WrappedTask]:
    """Create a new task

     Record a new task that the user intends to do. This will save the task in Dart for later access,
    search, etc. By default the created task will be assigned to the user, with a status of to-do, with
    no parent, in the Active dartboard. More information can be included in the description.

    Args:
        body (WrappedTaskCreate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[WrappedTask]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedTaskCreate,
) -> Optional[WrappedTask]:
    """Create a new task

     Record a new task that the user intends to do. This will save the task in Dart for later access,
    search, etc. By default the created task will be assigned to the user, with a status of to-do, with
    no parent, in the Active dartboard. More information can be included in the description.

    Args:
        body (WrappedTaskCreate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        WrappedTask
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedTaskCreate,
) -> Response[WrappedTask]:
    """Create a new task

     Record a new task that the user intends to do. This will save the task in Dart for later access,
    search, etc. By default the created task will be assigned to the user, with a status of to-do, with
    no parent, in the Active dartboard. More information can be included in the description.

    Args:
        body (WrappedTaskCreate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[WrappedTask]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedTaskCreate,
) -> Optional[WrappedTask]:
    """Create a new task

     Record a new task that the user intends to do. This will save the task in Dart for later access,
    search, etc. By default the created task will be assigned to the user, with a status of to-do, with
    no parent, in the Active dartboard. More information can be included in the description.

    Args:
        body (WrappedTaskCreate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        WrappedTask
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.wrapped_doc import WrappedDoc
from ...models.wrapped_doc_update import WrappedDocUpdate
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    body: WrappedDocUpdate,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": "/docs/{id}".format(
            id=id,
        ),
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, WrappedDoc]]:
    if response.status_code == 200:
        response_200 = WrappedDoc.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, WrappedDoc]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedDocUpdate,
) -> Response[Union[Any, WrappedDoc]]:
    """Update an existing doc

     Update certain properties of an existing doc. This will save the doc in Dart for later access,
    search, etc. Any properties that are not specified will not be changed.

    Args:
        id (str):
        body (WrappedDocUpdate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, WrappedDoc]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedDocUpdate,
) -> Optional[Union[Any, WrappedDoc]]:
    """Update an existing doc

     Update certain properties of an existing doc. This will save the doc in Dart for later access,
    search, etc. Any properties that are not specified will not be changed.

    Args:
        id (str):
        body (WrappedDocUpdate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, WrappedDoc]
    """

    return sync_detailed(
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedDocUpdate,
) -> Response[Union[Any, WrappedDoc]]:
    """Update an existing doc

     Update certain properties of an existing doc. This will save the doc in Dart for later access,
    search, etc. Any properties that are not specified will not be changed.

    Args:
        id (str):
        body (WrappedDocUpdate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, WrappedDoc]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WrappedDocUpdate,
) -> Optional[Union[Any, WrappedDoc]]:
    """Update an existing doc

     Update certain properties of an existing doc. This will save the doc in Dart for later access,
    search, etc. Any properties that are not specified will not be changed.

    Args:
        id (str):
        body (WrappedDocUpdate):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, WrappedDoc]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            body=body,
        )
    ).parsed



"""Contains methods for accessing the API"""

from .comment import create_comment, list_comments
from .config import get_config
from .dartboard import retrieve_dartboard
from .doc import create_doc, delete_doc, list_docs, retrieve_doc, update_doc
from .folder import retrieve_folder
from .task import create_task, delete_task, list_tasks, retrieve_task, update_task
from .view import retrieve_view



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_docs_o_item import ListDocsOItem
from ...models.paginated_concise_doc_list import PaginatedConciseDocList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    folder: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListDocsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    s: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["folder"] = folder

    params["folder_id"] = folder_id

    params["ids"] = ids

    params["in_trash"] = in_trash

    params["limit"] = limit

    json_o: Union[Unset, list[str]] = UNSET
    if not isinstance(o, Unset):
        json_o = []
        for o_item_data in o:
            o_item = o_item_data.value
            json_o.append(o_item)

    params["o"] = json_o

    params["offset"] = offset

    params["s"] = s

    params["text"] = text

    params["title"] = title

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/docs/list",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[PaginatedConciseDocList]:
    if response.status_code == 200:
        response_200 = PaginatedConciseDocList.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[PaginatedConciseDocList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    folder: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListDocsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    s: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Response[PaginatedConciseDocList]:
    """List all docs that the user has access to. This will return a list of docs, including the title,
    folder, text and others.

    Args:
        folder (Union[Unset, str]):
        folder_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListDocsOItem]]):
        offset (Union[Unset, int]):
        s (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedConciseDocList]
    """

    kwargs = _get_kwargs(
        folder=folder,
        folder_id=folder_id,
        ids=ids,
        in_trash=in_trash,
        limit=limit,
        o=o,
        offset=offset,
        s=s,
        text=text,
        title=title,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    folder: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListDocsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    s: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Optional[PaginatedConciseDocList]:
    """List all docs that the user has access to. This will return a list of docs, including the title,
    folder, text and others.

    Args:
        folder (Union[Unset, str]):
        folder_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListDocsOItem]]):
        offset (Union[Unset, int]):
        s (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedConciseDocList
    """

    return sync_detailed(
        client=client,
        folder=folder,
        folder_id=folder_id,
        ids=ids,
        in_trash=in_trash,
        limit=limit,
        o=o,
        offset=offset,
        s=s,
        text=text,
        title=title,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    folder: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListDocsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    s: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Response[PaginatedConciseDocList]:
    """List all docs that the user has access to. This will return a list of docs, including the title,
    folder, text and others.

    Args:
        folder (Union[Unset, str]):
        folder_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListDocsOItem]]):
        offset (Union[Unset, int]):
        s (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedConciseDocList]
    """

    kwargs = _get_kwargs(
        folder=folder,
        folder_id=folder_id,
        ids=ids,
        in_trash=in_trash,
        limit=limit,
        o=o,
        offset=offset,
        s=s,
        text=text,
        title=title,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    folder: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListDocsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    s: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Optional[PaginatedConciseDocList]:
    """List all docs that the user has access to. This will return a list of docs, including the title,
    folder, text and others.

    Args:
        folder (Union[Unset, str]):
        folder_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListDocsOItem]]):
        offset (Union[Unset, int]):
        s (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedConciseDocList
    """

    return (
        await asyncio_detailed(
            client=client,
            folder=folder,
            folder_id=folder_id,
            ids=ids,
            in_trash=in_trash,
            limit=limit,
            o=o,
            offset=offset,
            s=s,
            text=text,
            title=title,
        )
    ).parsed



from enum import Enum


class ListCommentsOItem(str, Enum):
    HIERARCHICAL = "hierarchical"
    PUBLISHED_AT = "published_at"
    PUBLISHED_AT_DESC = "-published_at"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.priority import Priority
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.custom_properties import CustomProperties


T = TypeVar("T", bound="TaskUpdate")


@_attrs_define
class TaskUpdate:
    """
    Attributes:
        id (str): The universal, unique ID of the task.
        title (Union[Unset, str]): The title, which is a short description of what needs to be done.
        parent_id (Union[None, Unset, str]): The universal, unique ID of the parent task. This can be null.
        dartboard (Union[Unset, str]): The full title of the dartboard, which is a project or list of tasks.
        type_ (Union[Unset, str]): The title of the type of the task.
        status (Union[Unset, str]): The status from the list of available statuses.
        description (Union[Unset, str]): A longer description of the task, which can include markdown formatting.
        assignees (Union[None, Unset, list[str]]): The names or emails of the users that the task is assigned to. Either
            this or assignee must be included, depending on whether the workspaces allows multiple assignees or not.
        assignee (Union[None, Unset, str]): The name or email of the user that the task is assigned to. Either this or
            assignees must be included, depending on whether the workspaces allows multiple assignees or not.
        tags (Union[Unset, list[str]]): Any tags that should be applied to the task, which can be used to filter and
            search for tasks. Tags are also known as labels or components and are strings that can be anything, but should
            be short and descriptive. This list can be empty.
        priority (Union[None, Priority, Unset]): The priority, which is a string that can be one of the specified
            options. This is used to sort tasks and determine which tasks should be done first.
        start_at (Union[None, Unset, str]): The start date, which is a date that the task should be started by in ISO
            format, like YYYY-MM-DD.
        due_at (Union[None, Unset, str]): The due date, which is a date that the task should be completed by in ISO
            format, like YYYY-MM-DD.
        size (Union[None, Unset, int, str]): The size, which represents the amount of work that needs to be done. This
            is used to determine how long the task will take to complete.
        custom_properties (Union['CustomProperties', None, Unset]): The custom properties, which is a dict of custom
            properties that are associated with the task.
    """

    id: str
    title: Union[Unset, str] = UNSET
    parent_id: Union[None, Unset, str] = UNSET
    dartboard: Union[Unset, str] = UNSET
    type_: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    assignees: Union[None, Unset, list[str]] = UNSET
    assignee: Union[None, Unset, str] = UNSET
    tags: Union[Unset, list[str]] = UNSET
    priority: Union[None, Priority, Unset] = UNSET
    start_at: Union[None, Unset, str] = UNSET
    due_at: Union[None, Unset, str] = UNSET
    size: Union[None, Unset, int, str] = UNSET
    custom_properties: Union["CustomProperties", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.custom_properties import CustomProperties

        id = self.id

        title = self.title

        parent_id: Union[None, Unset, str]
        if isinstance(self.parent_id, Unset):
            parent_id = UNSET
        else:
            parent_id = self.parent_id

        dartboard = self.dartboard

        type_ = self.type_

        status = self.status

        description = self.description

        assignees: Union[None, Unset, list[str]]
        if isinstance(self.assignees, Unset):
            assignees = UNSET
        elif isinstance(self.assignees, list):
            assignees = self.assignees

        else:
            assignees = self.assignees

        assignee: Union[None, Unset, str]
        if isinstance(self.assignee, Unset):
            assignee = UNSET
        else:
            assignee = self.assignee

        tags: Union[Unset, list[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        priority: Union[None, Unset, str]
        if isinstance(self.priority, Unset):
            priority = UNSET
        elif isinstance(self.priority, Priority):
            priority = self.priority.value
        else:
            priority = self.priority

        start_at: Union[None, Unset, str]
        if isinstance(self.start_at, Unset):
            start_at = UNSET
        else:
            start_at = self.start_at

        due_at: Union[None, Unset, str]
        if isinstance(self.due_at, Unset):
            due_at = UNSET
        else:
            due_at = self.due_at

        size: Union[None, Unset, int, str]
        if isinstance(self.size, Unset):
            size = UNSET
        else:
            size = self.size

        custom_properties: Union[None, Unset, dict[str, Any]]
        if isinstance(self.custom_properties, Unset):
            custom_properties = UNSET
        elif isinstance(self.custom_properties, CustomProperties):
            custom_properties = self.custom_properties.to_dict()
        else:
            custom_properties = self.custom_properties

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if title is not UNSET:
            field_dict["title"] = title
        if parent_id is not UNSET:
            field_dict["parentId"] = parent_id
        if dartboard is not UNSET:
            field_dict["dartboard"] = dartboard
        if type_ is not UNSET:
            field_dict["type"] = type_
        if status is not UNSET:
            field_dict["status"] = status
        if description is not UNSET:
            field_dict["description"] = description
        if assignees is not UNSET:
            field_dict["assignees"] = assignees
        if assignee is not UNSET:
            field_dict["assignee"] = assignee
        if tags is not UNSET:
            field_dict["tags"] = tags
        if priority is not UNSET:
            field_dict["priority"] = priority
        if start_at is not UNSET:
            field_dict["startAt"] = start_at
        if due_at is not UNSET:
            field_dict["dueAt"] = due_at
        if size is not UNSET:
            field_dict["size"] = size
        if custom_properties is not UNSET:
            field_dict["customProperties"] = custom_properties

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.custom_properties import CustomProperties

        d = dict(src_dict)
        id = d.pop("id")

        title = d.pop("title", UNSET)

        def _parse_parent_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        parent_id = _parse_parent_id(d.pop("parentId", UNSET))

        dartboard = d.pop("dartboard", UNSET)

        type_ = d.pop("type", UNSET)

        status = d.pop("status", UNSET)

        description = d.pop("description", UNSET)

        def _parse_assignees(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                assignees_type_0 = cast(list[str], data)

                return assignees_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        assignees = _parse_assignees(d.pop("assignees", UNSET))

        def _parse_assignee(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        assignee = _parse_assignee(d.pop("assignee", UNSET))

        tags = cast(list[str], d.pop("tags", UNSET))

        def _parse_priority(data: object) -> Union[None, Priority, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                priority_type_0 = Priority(data)

                return priority_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Priority, Unset], data)

        priority = _parse_priority(d.pop("priority", UNSET))

        def _parse_start_at(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        start_at = _parse_start_at(d.pop("startAt", UNSET))

        def _parse_due_at(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        due_at = _parse_due_at(d.pop("dueAt", UNSET))

        def _parse_size(data: object) -> Union[None, Unset, int, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int, str], data)

        size = _parse_size(d.pop("size", UNSET))

        def _parse_custom_properties(
            data: object,
        ) -> Union["CustomProperties", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                custom_properties_type_0 = CustomProperties.from_dict(data)

                return custom_properties_type_0
            except:  # noqa: E722
                pass
            return cast(Union["CustomProperties", None, Unset], data)

        custom_properties = _parse_custom_properties(d.pop("customProperties", UNSET))

        task_update = cls(
            id=id,
            title=title,
            parent_id=parent_id,
            dartboard=dartboard,
            type_=type_,
            status=status,
            description=description,
            assignees=assignees,
            assignee=assignee,
            tags=tags,
            priority=priority,
            start_at=start_at,
            due_at=due_at,
            size=size,
            custom_properties=custom_properties,
        )

        task_update.additional_properties = d
        return task_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.user import User
    from ..models.user_space_configuration_custom_property_checkbox_type_def import (
        UserSpaceConfigurationCustomPropertyCheckboxTypeDef,
    )
    from ..models.user_space_configuration_custom_property_dates_type_def import (
        UserSpaceConfigurationCustomPropertyDatesTypeDef,
    )
    from ..models.user_space_configuration_custom_property_multiselect_type_def import (
        UserSpaceConfigurationCustomPropertyMultiselectTypeDef,
    )
    from ..models.user_space_configuration_custom_property_number_type_def import (
        UserSpaceConfigurationCustomPropertyNumberTypeDef,
    )
    from ..models.user_space_configuration_custom_property_select_type_def import (
        UserSpaceConfigurationCustomPropertySelectTypeDef,
    )
    from ..models.user_space_configuration_custom_property_status_type_def import (
        UserSpaceConfigurationCustomPropertyStatusTypeDef,
    )
    from ..models.user_space_configuration_custom_property_text_type_def import (
        UserSpaceConfigurationCustomPropertyTextTypeDef,
    )
    from ..models.user_space_configuration_custom_property_time_tracking_type_def import (
        UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef,
    )
    from ..models.user_space_configuration_custom_property_user_type_def import (
        UserSpaceConfigurationCustomPropertyUserTypeDef,
    )


T = TypeVar("T", bound="UserSpaceConfiguration")


@_attrs_define
class UserSpaceConfiguration:
    """
    Attributes:
        today (datetime.date):
        user (User):
        dartboards (list[str]):
        folders (list[str]):
        types (list[str]):
        statuses (list[str]):
        assignees (list['User']):
        tags (list[str]):
        priorities (list[str]):
        sizes (Union[list[Union[int, str]], str]):
        custom_properties (list[Union['UserSpaceConfigurationCustomPropertyCheckboxTypeDef',
            'UserSpaceConfigurationCustomPropertyDatesTypeDef', 'UserSpaceConfigurationCustomPropertyMultiselectTypeDef',
            'UserSpaceConfigurationCustomPropertyNumberTypeDef', 'UserSpaceConfigurationCustomPropertySelectTypeDef',
            'UserSpaceConfigurationCustomPropertyStatusTypeDef', 'UserSpaceConfigurationCustomPropertyTextTypeDef',
            'UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef', 'UserSpaceConfigurationCustomPropertyUserTypeDef']]):
    """

    today: datetime.date
    user: "User"
    dartboards: list[str]
    folders: list[str]
    types: list[str]
    statuses: list[str]
    assignees: list["User"]
    tags: list[str]
    priorities: list[str]
    sizes: Union[list[Union[int, str]], str]
    custom_properties: list[
        Union[
            "UserSpaceConfigurationCustomPropertyCheckboxTypeDef",
            "UserSpaceConfigurationCustomPropertyDatesTypeDef",
            "UserSpaceConfigurationCustomPropertyMultiselectTypeDef",
            "UserSpaceConfigurationCustomPropertyNumberTypeDef",
            "UserSpaceConfigurationCustomPropertySelectTypeDef",
            "UserSpaceConfigurationCustomPropertyStatusTypeDef",
            "UserSpaceConfigurationCustomPropertyTextTypeDef",
            "UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef",
            "UserSpaceConfigurationCustomPropertyUserTypeDef",
        ]
    ]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.user_space_configuration_custom_property_checkbox_type_def import (
            UserSpaceConfigurationCustomPropertyCheckboxTypeDef,
        )
        from ..models.user_space_configuration_custom_property_dates_type_def import (
            UserSpaceConfigurationCustomPropertyDatesTypeDef,
        )
        from ..models.user_space_configuration_custom_property_multiselect_type_def import (
            UserSpaceConfigurationCustomPropertyMultiselectTypeDef,
        )
        from ..models.user_space_configuration_custom_property_number_type_def import (
            UserSpaceConfigurationCustomPropertyNumberTypeDef,
        )
        from ..models.user_space_configuration_custom_property_select_type_def import (
            UserSpaceConfigurationCustomPropertySelectTypeDef,
        )
        from ..models.user_space_configuration_custom_property_status_type_def import (
            UserSpaceConfigurationCustomPropertyStatusTypeDef,
        )
        from ..models.user_space_configuration_custom_property_text_type_def import (
            UserSpaceConfigurationCustomPropertyTextTypeDef,
        )
        from ..models.user_space_configuration_custom_property_time_tracking_type_def import (
            UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef,
        )

        today = self.today.isoformat()

        user = self.user.to_dict()

        dartboards = self.dartboards

        folders = self.folders

        types = self.types

        statuses = self.statuses

        assignees = []
        for assignees_item_data in self.assignees:
            assignees_item = assignees_item_data.to_dict()
            assignees.append(assignees_item)

        tags = self.tags

        priorities = self.priorities

        sizes: Union[list[Union[int, str]], str]
        if isinstance(self.sizes, list):
            sizes = []
            for sizes_type_1_item_data in self.sizes:
                sizes_type_1_item: Union[int, str]
                sizes_type_1_item = sizes_type_1_item_data
                sizes.append(sizes_type_1_item)

        else:
            sizes = self.sizes

        custom_properties = []
        for custom_properties_item_data in self.custom_properties:
            custom_properties_item: dict[str, Any]
            if isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyCheckboxTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyDatesTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyMultiselectTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyNumberTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertySelectTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyStatusTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyTextTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            elif isinstance(
                custom_properties_item_data,
                UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef,
            ):
                custom_properties_item = custom_properties_item_data.to_dict()
            else:
                custom_properties_item = custom_properties_item_data.to_dict()

            custom_properties.append(custom_properties_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "today": today,
                "user": user,
                "dartboards": dartboards,
                "folders": folders,
                "types": types,
                "statuses": statuses,
                "assignees": assignees,
                "tags": tags,
                "priorities": priorities,
                "sizes": sizes,
                "customProperties": custom_properties,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User
        from ..models.user_space_configuration_custom_property_checkbox_type_def import (
            UserSpaceConfigurationCustomPropertyCheckboxTypeDef,
        )
        from ..models.user_space_configuration_custom_property_dates_type_def import (
            UserSpaceConfigurationCustomPropertyDatesTypeDef,
        )
        from ..models.user_space_configuration_custom_property_multiselect_type_def import (
            UserSpaceConfigurationCustomPropertyMultiselectTypeDef,
        )
        from ..models.user_space_configuration_custom_property_number_type_def import (
            UserSpaceConfigurationCustomPropertyNumberTypeDef,
        )
        from ..models.user_space_configuration_custom_property_select_type_def import (
            UserSpaceConfigurationCustomPropertySelectTypeDef,
        )
        from ..models.user_space_configuration_custom_property_status_type_def import (
            UserSpaceConfigurationCustomPropertyStatusTypeDef,
        )
        from ..models.user_space_configuration_custom_property_text_type_def import (
            UserSpaceConfigurationCustomPropertyTextTypeDef,
        )
        from ..models.user_space_configuration_custom_property_time_tracking_type_def import (
            UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef,
        )
        from ..models.user_space_configuration_custom_property_user_type_def import (
            UserSpaceConfigurationCustomPropertyUserTypeDef,
        )

        d = dict(src_dict)
        today = isoparse(d.pop("today")).date()

        user = User.from_dict(d.pop("user"))

        dartboards = cast(list[str], d.pop("dartboards"))

        folders = cast(list[str], d.pop("folders"))

        types = cast(list[str], d.pop("types"))

        statuses = cast(list[str], d.pop("statuses"))

        assignees = []
        _assignees = d.pop("assignees")
        for assignees_item_data in _assignees:
            assignees_item = User.from_dict(assignees_item_data)

            assignees.append(assignees_item)

        tags = cast(list[str], d.pop("tags"))

        priorities = cast(list[str], d.pop("priorities"))

        def _parse_sizes(data: object) -> Union[list[Union[int, str]], str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                sizes_type_1 = []
                _sizes_type_1 = data
                for sizes_type_1_item_data in _sizes_type_1:

                    def _parse_sizes_type_1_item(data: object) -> Union[int, str]:
                        return cast(Union[int, str], data)

                    sizes_type_1_item = _parse_sizes_type_1_item(sizes_type_1_item_data)

                    sizes_type_1.append(sizes_type_1_item)

                return sizes_type_1
            except:  # noqa: E722
                pass
            return cast(Union[list[Union[int, str]], str], data)

        sizes = _parse_sizes(d.pop("sizes"))

        custom_properties = []
        _custom_properties = d.pop("customProperties")
        for custom_properties_item_data in _custom_properties:

            def _parse_custom_properties_item(
                data: object,
            ) -> Union[
                "UserSpaceConfigurationCustomPropertyCheckboxTypeDef",
                "UserSpaceConfigurationCustomPropertyDatesTypeDef",
                "UserSpaceConfigurationCustomPropertyMultiselectTypeDef",
                "UserSpaceConfigurationCustomPropertyNumberTypeDef",
                "UserSpaceConfigurationCustomPropertySelectTypeDef",
                "UserSpaceConfigurationCustomPropertyStatusTypeDef",
                "UserSpaceConfigurationCustomPropertyTextTypeDef",
                "UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef",
                "UserSpaceConfigurationCustomPropertyUserTypeDef",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_0 = UserSpaceConfigurationCustomPropertyCheckboxTypeDef.from_dict(data)

                    return custom_properties_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_1 = UserSpaceConfigurationCustomPropertyDatesTypeDef.from_dict(data)

                    return custom_properties_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_2 = UserSpaceConfigurationCustomPropertyMultiselectTypeDef.from_dict(
                        data
                    )

                    return custom_properties_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_3 = UserSpaceConfigurationCustomPropertyNumberTypeDef.from_dict(data)

                    return custom_properties_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_4 = UserSpaceConfigurationCustomPropertySelectTypeDef.from_dict(data)

                    return custom_properties_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_5 = UserSpaceConfigurationCustomPropertyStatusTypeDef.from_dict(data)

                    return custom_properties_item_type_5
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_6 = UserSpaceConfigurationCustomPropertyTextTypeDef.from_dict(data)

                    return custom_properties_item_type_6
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    custom_properties_item_type_7 = UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef.from_dict(
                        data
                    )

                    return custom_properties_item_type_7
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                custom_properties_item_type_8 = UserSpaceConfigurationCustomPropertyUserTypeDef.from_dict(data)

                return custom_properties_item_type_8

            custom_properties_item = _parse_custom_properties_item(custom_properties_item_data)

            custom_properties.append(custom_properties_item)

        user_space_configuration = cls(
            today=today,
            user=user,
            dartboards=dartboards,
            folders=folders,
            types=types,
            statuses=statuses,
            assignees=assignees,
            tags=tags,
            priorities=priorities,
            sizes=sizes,
            custom_properties=custom_properties,
        )

        user_space_configuration.additional_properties = d
        return user_space_configuration

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DocCreate")


@_attrs_define
class DocCreate:
    """
    Attributes:
        title (str): The title, which is a short description of the doc.
        folder (Union[Unset, str]): The full title of the folder, which is a project or list of docs.
        text (Union[Unset, str]): The full content of the doc, which can include markdown formatting.
    """

    title: str
    folder: Union[Unset, str] = UNSET
    text: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        folder = self.folder

        text = self.text

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "title": title,
            }
        )
        if folder is not UNSET:
            field_dict["folder"] = folder
        if text is not UNSET:
            field_dict["text"] = text

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        title = d.pop("title")

        folder = d.pop("folder", UNSET)

        text = d.pop("text", UNSET)

        doc_create = cls(
            title=title,
            folder=folder,
            text=text,
        )

        doc_create.additional_properties = d
        return doc_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListDocsOItem(str, Enum):
    CREATED_AT = "created_at"
    ORDER = "order"
    TITLE = "title"
    UPDATED_AT = "updated_at"
    VALUE_0 = "-created_at"
    VALUE_1 = "-order"
    VALUE_2 = "-title"
    VALUE_3 = "-updated_at"

    def __str__(self) -> str:
        return str(self.value)
